unit TestCltnMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Buttons, Cltn;

type
  TForm1 = class(TForm)
    StatusBar: TStatusBar;
    LeftPanel: TPanel;
    ItemGroup: TGroupBox;
    Label1: TLabel;
    ValueLabel: TLabel;
    Key: TEdit;
    Value: TEdit;
    AddButton: TButton;
    RemoveButton: TButton;
    FindButton: TButton;
    AddManyButton: TButton;
    CollectionKind: TRadioGroup;
    CreateButton: TButton;
    DestroyButton: TButton;
    RightPanel: TPanel;
    ContentsGroup: TGroupBox;
    EnumList: TListView;
    AutoTestButton: TButton;
    DebugButton: TButton;
    CountMatching: TButton;
    PerformanceTest: TButton;
    RemoveMany: TButton;
    ClearButton: TButton;
    SetValueButton: TButton;
    Edit1: TEdit;
    Quantity: TUpDown;
    StartAt: TUpDown;
    Edit2: TEdit;
    Label2: TLabel;
    Label3: TLabel;
    Panel1: TPanel;
    EnumGroup: TGroupBox;
    ResetButton: TBitBtn;
    Label4: TLabel;
    EnumValueLabel: TLabel;
    EnumKey: TEdit;
    EnumValue: TEdit;
    NextButton: TBitBtn;
    EnumRemove: TBitBtn;
    CollectionProperties: TPanel;
    IsMap: TCheckBox;
    IsSorted: TCheckBox;
    IsOrdered: TCheckBox;
    Duplicates: TRadioGroup;
    SortButton: TButton;
    procedure AddButtonClick(Sender: TObject);
    procedure RemoveButtonClick(Sender: TObject);
    procedure FindButtonClick(Sender: TObject);
    procedure AddManyButtonClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure CollectionKindClick(Sender: TObject);
    procedure CreateButtonClick(Sender: TObject);
    procedure DestroyButtonClick(Sender: TObject);
    procedure AutoTestButtonClick(Sender: TObject);
    procedure DebugButtonClick(Sender: TObject);
    procedure CountMatchingClick(Sender: TObject);
    procedure RemoveManyClick(Sender: TObject);
    procedure PerformanceTestClick(Sender: TObject);
    procedure ClearButtonClick(Sender: TObject);
    procedure SetValueButtonClick(Sender: TObject);
    procedure ResetButtonClick(Sender: TObject);
    procedure NextButtonClick(Sender: TObject);
    procedure EnumRemoveClick(Sender: TObject);
    procedure DuplicatesClick(Sender: TObject);
    procedure SortButtonClick(Sender: TObject);
  private
    { Private declarations }
    fCollection: ICollection;
    fEnumerator: IEnumerator;
    function GetMap: IMap;
    function GetEnumMap: IEnumMap;
    procedure CollectionChanged;
    procedure SetButtons;
    procedure EnumerateCollection;
    procedure EnumerateMap;
  public
    { Public declarations }
    property Collection: ICollection read fCollection write fCollection;
    property Enumerator: IEnumerator read fEnumerator write fEnumerator;
    property Map: IMap read GetMap;
    property EnumMap: IEnumMap read GetEnumMap;
  end;

var
  Form1: TForm1;

implementation

uses CltnHash, CltnArrays, CltnList, CltnRbTree, TreeDebugForm;

{$R *.DFM}

// Create a radio button for each kind of collection.
// The associated object is actually a class reference,
// to make it easy to create the collection object when
// the user clicks the Create button.
procedure TForm1.FormCreate(Sender: TObject);
begin
  CollectionKind.Items.AddObject('Hash Set', TObject(THashSet));
  CollectionKind.Items.AddObject('Hash Map', TObject(THashMap));
  CollectionKind.Items.AddObject('Tree Set', TObject(TRbTreeSet));
  CollectionKind.Items.AddObject('Tree Map', TObject(TRbTreeMap));
  CollectionKind.Items.AddObject('Linked List', TObject(TLinkedList));
  CollectionKind.Items.AddObject('Sorted List', TObject(TSortedList));
  CollectionKind.Items.AddObject('Array List', TObject(TArrayList));
  CollectionKind.Items.AddObject('Array Set', TObject(TArraySet));
  CollectionKind.Items.AddObject('Array Map', TObject(TArrayMap));
  CollectionKind.Items.AddObject('Sorted Array Set', TObject(TSortedArraySet));
  CollectionKind.Items.AddObject('Sorted Array Map', TObject(TSortedArrayMap));
  SetButtons;
end;

// Set the state of the buttons and other controls.
procedure TForm1.SetButtons;
var
  IsMap: Boolean;
  List: IList;
begin
  CreateButton.Enabled := CollectionKind.ItemIndex >= 0;
  DestroyButton.Enabled := Collection <> nil;
  AutoTestButton.Enabled := CreateButton.Enabled and not DestroyButton.Enabled;
  PerformanceTest.Enabled := AutoTestButton.Enabled;

  CreateButton.Default := not DestroyButton.Enabled;
  EnumGroup.Enabled := DestroyButton.Enabled;
  ItemGroup.Enabled := DestroyButton.Enabled;
  ContentsGroup.Enabled := DestroyButton.Enabled;
  AddButton.Default := ItemGroup.Enabled;

  SortButton.Enabled := (Collection <> nil) and (Collection.QueryInterface(IList, List) = S_OK);

  IsMap := (Collection <> nil) and Collection.IsMap;
  if IsMap then
    EnumList.ViewStyle := vsReport
  else
    EnumList.ViewStyle := vsList;

  SetValueButton.Enabled := IsMap;
  Value.Enabled := IsMap;
  ValueLabel.Enabled := IsMap;
  EnumValue.Enabled := IsMap;
  EnumValueLabel.Enabled := IsMap;
  Self.IsMap.Checked := IsMap;
  IsSorted.Checked := (Collection <> nil) and Collection.IsSorted;
  IsOrdered.Checked := (Collection <> nil) and Collection.IsOrdered;

  if Collection = nil then
  begin
    Duplicates.ItemIndex := -1;
    Duplicates.Enabled := False;
  end
  else
  begin
    Duplicates.ItemIndex := Ord(Collection.Duplicates);
    Duplicates.Enabled := True;
  end;

  DebugButton.Enabled := DebugButton.Tag <> 0;

  if Enumerator <> nil then
    NextButton.Enabled := Enumerator.HasNext;
  EnumRemove.Enabled := False;
end;

// Add an item to the collection.
procedure TForm1.AddButtonClick(Sender: TObject);
begin
  if Collection.IsMap then
    Map.Add(MakeString(Key.Text), MakeString(Value.Text))
  else
    Collection.Add(MakeString(Key.Text));
  CollectionChanged;
end;

// Try to remove an item from the collection.
procedure TForm1.RemoveButtonClick(Sender: TObject);
begin
  if Collection.IsMap then
    Map.RemoveKey(MakeString(Key.Text))
  else
    Collection.Remove(MakeString(Key.Text));
  CollectionChanged;
end;

// Find an item in the collection. Beep if the item is not found.
procedure TForm1.FindButtonClick(Sender: TObject);
var
  Found: ICollectible;
  Assoc: IAssociation;
  AssocValue: IUnknown;
begin
  if Collection.IsMap then
  begin
    if Map.FindKey(MakeString(Key.Text), Assoc) then
    begin
      Assoc.GetValue(AssocValue);
      Found := AssocValue as ICollectible;
    end
    else
      Found := nil
  end
  else if not Collection.Find(MakeString(Key.Text), Found) then
    Found := nil;

  if Found = nil then
  begin
    Value.Clear;
    MessageBeep(Mb_IconExclamation);
    StatusBar.SimpleText := 'Item not found';
  end
  else
    Value.Text := (Found as IString).Value;
end;

procedure TForm1.EnumerateMap;
var
  Enum: IEnumMap;
  Assoc: IAssociation;
  Key: ICollectible;
  Value: IUnknown;
begin
  Map.Enumerator(Enum);
  while Enum.Next(Assoc) do
  begin
    Assoc.GetKey(Key);
    Assoc.GetValue(Value);
    with EnumList.Items.Add do
    begin
      Caption := (Key as IString).Value;
      SubItems.Add((Value as IString).Value);
    end;
  end;
end;

procedure TForm1.EnumerateCollection;
var
  Item: ICollectible;
  Enum: IEnumerator;
begin
  Collection.Enumerator(Enum);
  while Enum.Next(Item) do
    EnumList.Items.Add.Caption := (Item as IString).Value
end;

// When the collection changes, due to adding or removing an item,
// redisplay the entire collection, and update the status bar.
procedure TForm1.CollectionChanged;
begin
  EnumList.Items.Clear;
  if Collection = nil then
    StatusBar.SimpleText := 'no collection'
  else
  begin
    StatusBar.SimpleText := Format('Count: %d', [Collection.Count]);
    if Collection.IsMap then
      EnumerateMap
    else
      EnumerateCollection;
  end;
  NextButton.Enabled := (Enumerator <> nil) and Enumerator.HasNext;
  EnumRemove.Enabled := False;
end;

// Clear the collection
procedure TForm1.ClearButtonClick(Sender: TObject);
begin
  Collection.Clear;
  CollectionChanged;
end;

// Add many items at once.
procedure TForm1.AddManyButtonClick(Sender: TObject);
var
  I, N: Integer;
begin
  N := StartAt.Position;
  for I := 1 to Quantity.Position do
  begin
    if Collection.IsMap then
      Map.Add(MakeString('Key'+IntToStr(N)), MakeString('Value'+IntToStr(N)))
    else
      Collection.Add(MakeString('Key'+IntToStr(N)));
    Inc(N);
  end;
  CollectionChanged;
  StartAt.Position := N;
end;

// Remove many items whose names are the items most recently
// added. If any item is not present, the collection raises an
// exception, and the entire removal process stops.
procedure TForm1.RemoveManyClick(Sender: TObject);
var
  I, N: Integer;
begin
  for I := 1 to Quantity.Position do
  begin
    N := StartAt.Position - Quantity.Position + I - 1;
    if Collection.IsMap then
      Map.RemoveKey(MakeString('Key'+IntToStr(N)))
    else
      Collection.Remove(MakeString('Key'+IntToStr(N)));
  end;
  CollectionChanged;
end;

// Count the number of items that match the key.
procedure TForm1.CountMatchingClick(Sender: TObject);
var
  Count: Integer;
begin
  if Collection.IsMap then
    Count := Collection.CountMatching(MakeAssociation(MakeString(Key.Text)) as ICollectible)
  else
    Count := Collection.CountMatching(MakeString(Key.Text));
  ShowMessage(IntToStr(Count));
  StatusBar.SimpleText := Format('Matching count: %d', [Count]);
end;

// Update the buttons, etc., when the user selects a collection kind.
procedure TForm1.CollectionKindClick(Sender: TObject);
begin
  SetButtons;
end;

type
  TExposeTreeMap = class(TRbTreeMap);
  TExposeTreeSet = class(TRbTreeSet);

// Create a new collection.
procedure TForm1.CreateButtonClick(Sender: TObject);
var
  Tree: TRbTree;
  CollectionClass: TCollectionClass;
  Coll: TCollection;
begin
  CollectionClass := TCollectionClass(CollectionKind.Items.Objects[CollectionKind.ItemIndex]);
  Coll := CollectionClass.Create;
  // If the collection is a red-black tree, save a reference to the
  // tree object as the Tag for DebugButton. When the user clicks the
  // button, cast the Tag back to a TRbTree pointer to show the tree.
  if Coll is TRbTreeMap then
    Tree := TExposeTreeMap(Coll).Tree
  else if Coll is TRbTreeSet then
    Tree := TExposeTreeSet(Coll).Tree
  else
    Tree := nil;
  DebugButton.Tag := Integer(Tree);

  Collection := Coll;
  Collection.Enumerator(fEnumerator);
  CollectionChanged;
  SetButtons;
end;

// Destroy the collection.
procedure TForm1.DestroyButtonClick(Sender: TObject);
begin
  Enumerator := nil;
  Collection := nil;
  CollectionChanged;
  SetButtons;
end;

// Run a simple automatic test. This test is amazingly effective
// at finding stupid errors during development, but is nowhere
// close to being complete.
procedure TForm1.AutoTestButtonClick(Sender: TObject);
var
  Enum: IEnumerator;
  CollClass: TCollectionClass;
  Assoc: IAssociation;
  Key: ICollectible;
  Value: IUnknown;
begin
  CollClass := TCollectionClass(CollectionKind.Items.Objects[CollectionKind.ItemIndex]);
  Collection := CollClass.Create;
  try
    Assert(Collection.IsEmpty);

    if Collection.IsMap then
      Map.Add(MakeString('First'), MakeString('First Value'))
    else
      Collection.Add(MakeString('First'));
    Assert(Collection.Count = 1);

    if Collection.IsMap then
      Map.Add(MakeString('Second'), MakeString('Second Value'))
    else
      Collection.Add(MakeString('Second'));
    Assert(Collection.Count = 2);

    if Collection.IsMap then
    begin
      Assert((Map[MakeString('Second')] as IString).Value = 'Second Value');
      Assert(Map.FindValue(MakeString('Second Value'), Assoc));
      Assoc.GetValue(Value);
      Assert((Value as IString).Value = 'Second Value');
    end
    else
    begin
      Assert(Collection.Find(MakeString('Second'), Key));
      Assert((Key as IString).Value = 'Second');
    end;

    if Collection.IsMap then
    begin
      Assert(not Map.FindKey(MakeString('Third'), Assoc));
      Assert(not Map.FindValue(MakeString('Third'), Assoc));
      Assert(not Map.ContainsKey(MakeString('Third')));
      Assert(not Map.ContainsValue(MakeString('Third')));
    end
    else
    begin
      Assert(not Collection.Find(MakeString('Third'), Key));
      Assert(not Collection.Contains(MakeString('Third')));
    end;


    Collection.Enumerator(Enum);
    Assert(Enum.HasNext);
    Enum.Skip;
    Enum.Remove;
    Assert(Enum.HasNext);
    Assert(Collection.Count = 1);
    Enum.Skip;
    Assert(not Enum.HasNext);
    Enum.Remove;
    Assert(not Enum.HasNext);

    Assert(Collection.IsEmpty);

    if Collection.IsMap then
    begin
      Assert(not Map.ContainsKey(MakeString('First')));
      Assert(not Map.ContainsValue(MakeString('First')));
    end
    else
      Assert(not Collection.Contains(MakeString('First')));

    if Collection.IsMap then
      Map.Add(MakeString('Third'), MakeString('Third Value'))
    else
      Collection.Add(MakeString('Third'));

    Collection.Enumerator(Enum);
    if Collection.IsMap then
    begin
      Assert((Enum as IEnumMap).Next(Assoc));
      Assoc.GetKey(Key);
      Assert((Key as IString).Value = 'Third')
    end
    else
    begin
      Assert(Enum.Next(Key));
      Assert((Key as IString).Value = 'Third');
    end;

    Assert(Collection.Count = 1);
    if Collection.IsMap then
      Map.RemoveKey(MakeString('Third'))
    else
      Collection.Remove(MakeString('Third'));

    Assert(Collection.Count = 0);
    if Collection.IsMap then
    begin
      Assert(not Map.ContainsKey(MakeString('Third')));
      Assert(not Map.ContainsValue(MakeString('Third')));
    end
    else
      Assert(not Collection.Contains(MakeString('Third')));

    Collection := nil;
    StatusBar.SimpleText := 'Test passed';
  except
    StatusBar.SimpleText := 'Test FAILED';
  end;
end;

// Return the collection as a map.
function TForm1.GetMap: IMap;
begin
  Result := Collection as IMap;
end;

// If the collection is a red-black tree, display a debug window
// that shows the tree structure.
procedure TForm1.DebugButtonClick(Sender: TObject);
begin
  TTreeForm.ShowTree(TRbTree(TComponent(Sender).Tag));
end;

// Perform a simple performance test. Prompt the user for the number
// of items to create, then add that many items to the collection.
// Each item is a random integer.
// Then search for random keys and if found, remove them. Some key
// values will be found and some will not.

procedure TForm1.PerformanceTestClick(Sender: TObject);
var
  CollectionClass: TCollectionClass;
  PerfLimit: Integer;
  PerfString: string;
  TimeStart, TimeDiff: DWORD;
  Key: ICollectible;
  Assoc: IAssociation;
  Found: ICollectible;
  I: Integer;
begin
  PerfString := '10000';
  if not InputQuery(Caption, 'Collection Count:', PerfString) then
    Exit;
  PerfLimit := StrToInt(PerfString);

  Screen.Cursor := crHourglass;
  try
    CollectionClass := TCollectionClass(CollectionKind.Items.Objects[CollectionKind.ItemIndex]);
    Collection := CollectionClass.Create;

    TimeStart := GetTickCount;
    for I := 1 to PerfLimit do
      if Collection.IsMap then
        Map.Add(MakeInteger(Random(PerfLimit)), MakeInteger(Random(PerfLimit)))
      else
        Collection.Add(MakeInteger(Random(PerfLimit)));

    for I := 1 to PerfLimit do
    begin
      Key := MakeInteger(Random(PerfLimit));
      if not Collection.IsMap then
      begin
        if not Collection.Find(Key, Found) then
          Found := nil;
      end
      else if Map.FindKey(Key, Assoc) then
        Found := Assoc as ICollectible
      else
        Found := nil;
      if Found <> nil then
        Collection.Remove(Found);
    end;

    Collection := nil;
    CollectionChanged;
    TimeDiff := GetTickCount - TimeStart;
    StatusBar.SimpleText := Format('%d msec', [TimeDiff]);
  finally
    Screen.Cursor := crDefault;
  end;
end;

// Set a value in a map.
procedure TForm1.SetValueButtonClick(Sender: TObject);
begin
  Map[MakeString(Key.Text)] := MakeString(Value.Text);
  CollectionChanged;
end;

function TForm1.GetEnumMap: IEnumMap;
begin
  Result := Enumerator as IEnumMap;
end;

// Reset the enumerator.
procedure TForm1.ResetButtonClick(Sender: TObject);
begin
  Enumerator.Reset;
  NextButton.Enabled := Enumerator.HasNext;
  EnumRemove.Enabled := False;
  EnumKey.Clear;
  EnumValue.Clear;
end;

// Get the next item from the enumerator.
procedure TForm1.NextButtonClick(Sender: TObject);
var
  Item: ICollectible;
  Key: ICollectible;
  Value: IUnknown;
begin
  Assert(Enumerator.Next(Item));
  if Collection.IsMap then
  begin
    (Item as IAssociation).GetKey(Key);
    (Item as IAssociation).GetValue(Value);
    EnumKey.Text := (Key as IString).Value;
    EnumValue.Text := (Value as IString).Value;
  end
  else
    EnumKey.Text := (Item as IString).Value;

  NextButton.Enabled := Enumerator.HasNext;
  EnumRemove.Enabled := True;
end;

// Remove the item that was most recently retrieved from the enumerator.
procedure TForm1.EnumRemoveClick(Sender: TObject);
begin
  Enumerator.Remove;
  CollectionChanged;
  EnumKey.Clear;
  EnumValue.Clear;
end;

procedure TForm1.DuplicatesClick(Sender: TObject);
var
  Dup: TDuplicates;
begin
  Dup := TDuplicates(Duplicates.ItemIndex);
  if (Collection <> nil) and (Dup <> Collection.Duplicates) then
    Collection.Duplicates := Dup;
end;

procedure TForm1.SortButtonClick(Sender: TObject);
begin
  (Collection as IList).Sort;
  CollectionChanged;
end;


end.
