unit TreeDebugForm;

interface

// Debug a red-black tree by showing the internal tree structure.
// Visibly display an image for red and black nodes.

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ComCtrls, ImgList, CltnRbTree;

type
  TTreeForm = class(TForm)
    TreeView: TTreeView;
    ImageList: TImageList;
  private
    { Private declarations }
    fTree: TRbTree;
    procedure SetTree(Tree: TRbTree);
  public
    { Public declarations }
    class procedure ShowTree(Tree: TRbTree);
    property Tree: TRbTree read fTree write SetTree;
  end;

var
  TreeForm: TTreeForm;

implementation

uses Cltn;

{$R *.DFM}

type
  TExposeTree = class(TRbTree)
  end;
  TExposeNode = class(TRbNode)
  end;

{ TTreeForm }

procedure TTreeForm.SetTree(Tree: TRbTree);

  procedure WalkTree(RbNode: TRbNode; ViewNode: TTreeNode);
  var
    Item: ICollectible;
    Assoc: IAssociation;
  begin
    if RbNode <> TExposeTree(Tree).Leaf then
    begin
      // Get the string item from the node. If the collection is a map,
      // the node holds an association, and the string is the key.
      Item := TExposeNode(RbNode).Item;
      if Item.QueryInterface(IAssociation, Assoc) = S_OK then
        Assoc.GetKey(Item);
      ViewNode := TreeView.Items.AddChild(ViewNode, (Item as IString).Value);
      if TExposeNode(RbNode).IsRed then
        ViewNode.ImageIndex := 0
      else
        ViewNode.ImageIndex := 1;
      ViewNode.SelectedIndex := ViewNode.ImageIndex;

      WalkTree(TExposeNode(RbNode).Left, ViewNode);
      WalkTree(TExposeNode(RbNode).Right, ViewNode);
    end;
  end;

begin
  fTree := Tree;
  TreeView.Items.Clear;
  WalkTree(TExposeTree(Tree).Root, nil);
  TreeView.FullExpand;
end;

class procedure TTreeForm.ShowTree(Tree: TRbTree);
var
  Form: TTreeForm;
begin
  if Tree = nil then
    MessageBeep(Mb_IconExclamation)
  else
  begin
    Form := TTreeForm.Create(Application);
    try
      Form.Tree := Tree;
      Form.ShowModal;
    finally
      Form.Free;
    end;
  end;
end;

end.
